package com.cirle.scientific.po;

import io.swagger.annotations.ApiModel;
import io.swagger.annotations.ApiModelProperty;
import lombok.Data;

import javax.persistence.*;
import java.io.Serializable;
import java.util.Calendar;

@Entity
@Table(name = "qkj_live")
@Data
@ApiModel
public class QkjLive implements Serializable {

	private static final long serialVersionUID = -4530263139965671680L;

	@Id
	@GeneratedValue(strategy = GenerationType.IDENTITY)
	private Long id;

	@ApiModelProperty(value = "直播id")
	private String liveId;

	@ApiModelProperty(value = "直播标题")
	private String liveTitle;

	@ApiModelProperty(value = "支付价格")
	private Double price;

	@ApiModelProperty(value = "是否支持积分兑换  0：不支持  1：支持")
	private Integer pointFlag=0;

	@ApiModelProperty(value = "积分兑换所需积分(如果支持积分兑换此值大于0)")
	private Integer point=0;

	@ApiModelProperty(value = "购买后有效时间（单位：天）")
	private Integer durationValidity = 365;

	@ApiModelProperty(value = "直播详情")
	private String liveContent;

	@ApiModelProperty(value = "直播封面图")
	private String liveCoverImg;

	@ApiModelProperty(value = "开始时间")
	private Calendar liveBeginTime;

	@ApiModelProperty(value = "结束时间")
	private Calendar liveEndTime;

	@ApiModelProperty(value = "直播类型，1-普通直播，2-系列直播，3-三方推流直播")
	private Integer liveType=1;

	@ApiModelProperty(value = "直播域名")
	private String liveDomain;

	@ApiModelProperty(value = "直播地址")
	private String liveUri;

	@ApiModelProperty(value = "直播推流地址")
	private String liveRtmpUrl;

	@ApiModelProperty(value = "直播观看地址")
	private String livePlayUrl;

	@ApiModelProperty(value = "直播状态，0-未开始，1-进行中，2-已结束，3-可回看")
	private Integer liveStatus=0;

	@ApiModelProperty(value = "参与用户数")
	private Integer joinMembers=0;

	@ApiModelProperty(value = "在线用户数")
	private Integer onlineMembers=0;

	@ApiModelProperty(value = "分享标题")
	private String shareTitle;

	@ApiModelProperty(value = "分享描述")
	private String shareDesc;

	@ApiModelProperty(value = "分享图标")
	private String sharePic;

	@ApiModelProperty(value = "是否置顶，0-不置顶，1-已置顶")
	private Integer istop;

	@ApiModelProperty(value = "视频库id")
	private Integer vodId;

	@ApiModelProperty(value = "直播回看地址")
	private String backviewUrl;

	@ApiModelProperty(value = "发布状态， 0-草稿 1-正常，2-已删除")
	private Integer status;

	private Calendar createTime;

	@ApiModelProperty(value = "系列直播中上架子直播数量")
	private Integer liveSonCount = 0;

	@ApiModelProperty(value = "真实预约数")
	private Integer remindMembers = 0;

	@ApiModelProperty(value = "假预约人数")
	private Integer vremindMembers = 0;

	@ApiModelProperty(value = "真实购买数量")
	private Integer purchasedMembers = 0;

	@ApiModelProperty(value = "假购买数量")
	private Integer vpurchasedMembers = 0;

	@ApiModelProperty(value = "是否需要实名认证  0:不需要 1:需要")
	private Integer smrzFlag = 0;

	@ApiModelProperty(value = "是否需要医师认证  0:不需要 1:需要")
	private Integer ysrzFlag = 0;

	@ApiModelProperty(value = "是否需要问卷调查  0:不需要 1:需要")
	private Integer questionFlag = 0;

	@ApiModelProperty(value = "需要进行的认证的类型 如果没有则不需要 1:以前的医师认证 2：以前的实名认证 3：欧佳龙医师认证 4：欧佳龙实名 5：不需要")
	private Integer attestType = 5;

	@ApiModelProperty(value = "是否对外售卖  0、可售卖  1、不对外售卖 （专区）")
	private Integer isSell = 0;

	@ApiModelProperty(value = "云展厅")
	private String showHall;

	@ApiModelProperty(value = "三方推m3u8流地址")
	@Column(name = "push_stream_m3u8")
	private String pushStreamM3u8;

	@ApiModelProperty(value = "三方推flv流地址")
	private String pushStreamFlv;

	@ApiModelProperty(value = "色卡")
	private String colorSwatches;

	@ApiModelProperty(value = "色卡")
	private String liveTab;

	@ApiModelProperty(value = "视频的时长")
	private Integer vodLength;

	@Column(name = "interact_monthly_img")
	@ApiModelProperty(value = "互动榜单背景图")
	private String interactMonthlyImg;

	@Column(name = "scroll_text")
	@ApiModelProperty(value = "滚动文字")
	private String scrollText;

	@Column(name = "color_text_swatches")
	@ApiModelProperty(value = "滚动文字配色")
	private String colorTextSwatches;

	@Column(name = "ago_questionnaire")
	@ApiModelProperty(value = "会前问卷")
	private String agoQuestionnaire;

	@Column(name = "cook_questionnaire")
	@ApiModelProperty(value = "会后问卷")
	private String  cookQuestionnaire;

	@Column(name = "question_tooltip")
	@ApiModelProperty(value = "问卷标题")
	private String questionTooltip;

	/** 学习时长 */
	@Transient
	private Integer learnTime;

	@ApiModelProperty(value = "是否在列表显示 0-隐藏  1-显示（普通直播）")
	private Integer isVisiable;

	@ApiModelProperty(value = "调查问卷是否需要审核 0：不需要 1：需要")
	private Integer needAuditing;

}
